/***************************************************
* By CCSC compiler
*  File Name: 20210114_K653_MK158C_1503.c
 * rev 1.0 20210114
 * rev 2.0 20210207
*  Description: 
 * RA5 O TX FOR MK156 CONTROL
 * RA4 I NIGHT=0
 * RA3 I SW4-8
 * RA2 I SW4-4
 * RA1 I SW4-2
 * RA0 I SW4-1
 * RC5 O LED
 * RC4 I BUSY FOR MK156
 * RC3 ADC LDR
 * RC2 O POWER
 * RC1 O POWER FOR LDR
 * RC0 I NA
*  COPYRIGHT 2021 MYCOMKITS.COM, owned by CNET LIMITED
* The copyright of this program belongs to the producer "MYCOMKITS.COM management company CNET LIMITED". 
* We do not abandon copyright, but you can use and change the program in learning using this program. 
***************************************************/
//
#include <16f1503.h>
#DEVICE ADC=8
#fuses	INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT,
#fuses	NOCLKOUT, NOWRT
#use	delay(CLOCK = 4000000)
#use	rs232(BAUD=9600, XMIT=PIN_A5)
//
#use	fast_io(a)
#use	fast_io(c)
//
#define Amode 0xDF			//port A all input except A5
#define Cmode 0x19			//port C all input, except C1,2
#define MODE PIN_A4
#define BUSY PIN_C4 // active high
#define PIR PIN_A2 // active high
#define LDR_POWER PIN_C1 // active high
#define MK156_POWER PIN_C2 // active high
#define LED PIN_C5 // active high

// variables
int value;
int file_number;
int enable_flag = 0;
int wakeup=0;
// prototyping
#separate
void get_sensor();
#separate
void initializing();
#separate
void mode_check();
#separate
void play_one();
#separate
void play_stop();
#separate
void get_number();

// main function
void main()
{
    initializing();

    // main loop
    while(1)
    {
        if(wakeup==1)
        {
            disable_interrupts(GLOBAL);
            port_a_pullups(0x1B);   //except RA2,5
            delay_ms(30);
            mode_check();   //day or night
            output_high(LED);
            output_high(PIN_A5); //RS232 on

            if(enable_flag == 1)
            {
                output_high(MK156_POWER);    //power on
                if(input(BUSY) == 0)
                {
                    
                    delay_ms(500);
                    get_number();
                    //specify file
                    putc(0x7E);
                    putc(0x04);
                    putc(0x42);
                    putc(0x01); //folder 001
                    putc(file_number);
                    putc(0xEF);
                    delay_ms(100);
                   play_one();

                   while(input(BUSY) == 1) {}
                   //
                   output_low(MK156_POWER);    //power off
                   
                }
            }
        }
        
        port_a_pullups(0x00);   //except RA2,5
        output_low(LDR_POWER);
        output_low(MK156_POWER);    //power off
        output_low(PIN_A5); //RS232 off
        
        while(input(PIN_A2)==1) {}
        output_low(LED);
        wakeup=0;
        clear_interrupt(INT_EXT);
        enable_interrupts(GLOBAL);
        sleep();
        
    }
}
//
#separate
void get_sensor()
{
    set_adc_channel(7);	//RC3, red, must wait 65u
    delay_us(100);
    value = read_adc();	//0 to 255
    if(value < 5) value = 0;	// 2 to 255
}
//
#separate
void initializing()
{
	set_tris_a(Amode);	//
	set_tris_c(Cmode);	//
	//
	setup_comparator(NC_NC_NC_NC);
	// A/D converter initialize
	setup_adc_ports(sAN7 | VSS_VDD);	// RC3, gnd to 5v
	setup_adc(ADC_CLOCK_DIV_8);			// 2usec
    
    output_low(LDR_POWER);  //active high
    output_low(MK156_POWER);   //active high
    output_low(LED);
    
    enable_interrupts(INT_EXT);
    ext_int_edge(L_TO_H);
    enable_interrupts(GLOBAL);

}

#INT_EXT
void ext_isr(void)
{
    delay_ms(30);
    wakeup=1;
}
//
#separate
void mode_check()
{
    if(input(MODE) == 0)    // night
    {
        output_high(LDR_POWER);
        delay_ms(100);
        get_sensor();
        if(value < 0x7F) enable_flag = 1;
        else enable_flag = 0;
    }
    else
    {
        enable_flag = 1;
        output_low(LDR_POWER);  //not available
    }
}
//
#separate
void play_one()
 {
    putc(0x7E);
    putc(0x02);
    putc(0x01);
    putc(0xEF);
    delay_ms(1000);
    //
}
//
#separate
void play_stop()
{
    putc(0x7E);
    putc(0x02);
    putc(0x0E); // stop
    putc(0xEF);
    delay_ms(10);
}
//
#separate
void get_number()
{
    file_number = input_a() & 0x0B; // 0 to 11
    if(input(PIN_C0)==1) file_number = file_number + 4;
    if(file_number == 0) file_number = 16;
}


